#include "cube.hh"

#include <algorithm>

namespace {
  unsigned bitmask(std::initializer_list<int> bits) {
    unsigned mask = 0;
    for(int i : bits) {
      mask |= 1 << i;
    }
    return mask;
  }
}

cube::cube(double x, double y, double z, double a)
  : pts { point3d(x    , y    , z),
          point3d(x + a, y    , z),
          point3d(x    , y + a, z),
          point3d(x + a, y + a, z),
          point3d(x    , y    , z + a),
          point3d(x + a, y    , z + a),
          point3d(x    , y + a, z + a),
          point3d(x + a, y + a, z + a)
    }
{
}

void cube::rotate(point3d const &pivot, double anglex, double angley, double anglez) {
  for(auto &pt : pts) {
    ::rotate(pt, pivot, anglex, angley, anglez);
  }
}

void cube::apply_line(flipdot::image &img, unsigned back_mask, int x, int y) {
  unsigned used_mask = bitmask({x, y});
  if((back_mask & used_mask) == 0) {
    draw_line(img, pts[x], pts[y]);
  }
}

void cube::render(flipdot::image &img, bool opaque, bool with_faces) {
  unsigned mask = 0;

  if(opaque) {
    auto pt_back = std::max_element(std::begin(pts), std::end(pts), [](point3d const &lhs, point3d const &rhs) { return lhs.z() < rhs.z(); });
    mask = 1 << std::distance(std::begin(pts), pt_back);
  }

  apply_line(img, mask, 0, 1);
  apply_line(img, mask, 0, 2);
  apply_line(img, mask, 0, 4);
  apply_line(img, mask, 1, 3);
  apply_line(img, mask, 1, 5);
  apply_line(img, mask, 2, 3);
  apply_line(img, mask, 2, 6);
  apply_line(img, mask, 3, 7);
  apply_line(img, mask, 4, 5);
  apply_line(img, mask, 4, 6);
  apply_line(img, mask, 5, 7);
  apply_line(img, mask, 6, 7);

  if(with_faces) {

    unsigned face_masks[] = {
      bitmask({0, 1, 2, 3}),
      bitmask({1, 3, 5, 7}),
      bitmask({0, 1, 4, 5}),
      bitmask({2, 3, 6, 7}),
      bitmask({0, 2, 4, 6}),
      bitmask({4, 5, 6, 7})
    };

    if((face_masks[0] & mask) == 0) {
      draw_point(img, interpolate(pts[0], pts[3], 0.5));
    }

    if((face_masks[1] & mask) == 0) {
      draw_point(img, interpolate(pts[3], pts[5], 0.25));
      draw_point(img, interpolate(pts[3], pts[5], 0.75));
    }

    if((face_masks[2] & mask) == 0) {
      draw_point(img, interpolate(pts[0], pts[5], 0.25));
      draw_point(img, interpolate(pts[0], pts[5], 0.5 ));
      draw_point(img, interpolate(pts[0], pts[5], 0.75));
    }

    if((face_masks[3] & mask) == 0) {
      draw_point(img, interpolate(pts[2], pts[7], 0.25));
      draw_point(img, interpolate(pts[2], pts[7], 0.75));
      draw_point(img, interpolate(pts[3], pts[6], 0.25));
      draw_point(img, interpolate(pts[3], pts[6], 0.75));
    }

    if((face_masks[4] & mask) == 0) {
      draw_point(img, interpolate(pts[0], pts[6], 0.25));
      draw_point(img, interpolate(pts[0], pts[6], 0.5 ));
      draw_point(img, interpolate(pts[0], pts[6], 0.75));
      draw_point(img, interpolate(pts[2], pts[4], 0.25));
      draw_point(img, interpolate(pts[2], pts[4], 0.75));
    }

    if((face_masks[5] & mask) == 0) {
      point3d ul = interpolate(pts[4], pts[7], 0.25);
      point3d lr = interpolate(pts[4], pts[7], 0.75);
      point3d ur = interpolate(pts[5], pts[6], 0.25);
      point3d ll = interpolate(pts[5], pts[6], 0.75);

      draw_point(img, ul);
      draw_point(img, lr);
      draw_point(img, ur);
      draw_point(img, ll);
      draw_point(img, interpolate(ul, ll, 0.5));
      draw_point(img, interpolate(ur, lr, 0.5));
    }
  }
}
