#include "parts.hh"
#include "strips.hh"
#include "alsa.h"

#include <flipdot/flipdot.hh>

#include <boost/chrono.hpp>
#include <boost/thread.hpp>

#include <fstream>
#include <iterator>
#include <stdexcept>
#include <vector>

void hardcut(flipdot::display &disp) {
  flipdot::framebuffer fb1 = disp.empty_framebuffer();
  flipdot::framebuffer fb2 = fb1;

  std::uint16_t data[] = { 0xaaaa, 0x5555 };

  for(std::size_t i = 0; i < fb1.size(); ++i) {
    fb1[i] = data[ i      % 2];
    fb2[i] = data[(i + 1) % 2];
  }

  disp.flush(fb1);
  boost::this_thread::sleep(boost::posix_time::milliseconds(100));
  disp.flush(fb2);
  boost::this_thread::sleep(boost::posix_time::milliseconds(100));
}

void softcut_black(flipdot::display &disp) {
  flipdot::framebuffer fb_black  = disp.empty_framebuffer();
  flipdot::framebuffer fb_check  = fb_black;

  std::uint16_t data[] = { 0xaaaa, 0x5555 };

  for(std::size_t i = 0; i < fb_black.size(); ++i) {
    fb_check[i] = data[i % 2];
    fb_black[i] = 0xffff;
  }

  disp.flush(fb_check);
//  boost::this_thread::sleep(boost::posix_time::milliseconds(80));
  disp.flush(fb_black);
//  boost::this_thread::sleep(boost::posix_time::milliseconds(80));
}

void softcut_white(flipdot::display &disp) {
  flipdot::framebuffer fb_white  = disp.empty_framebuffer();
  flipdot::framebuffer fb_check  = fb_white;

  std::uint16_t data[] = { 0x5555, 0xaaaa };

  for(std::size_t i = 0; i < fb_white.size(); ++i) {
    fb_check[i] = data[i % 2];
  }

  disp.flush(fb_check);
//  boost::this_thread::sleep(boost::posix_time::milliseconds(80));
  disp.flush(fb_white);
//  boost::this_thread::sleep(boost::posix_time::milliseconds(80));
}

void part_intro(flipdot::display &disp, boost::chrono::time_point<boost::chrono::system_clock> const &start) {
  auto next = start;

  for(std::size_t i = 0; i < 8; ++i) {
    softcut_black(disp);
    next = next + boost::chrono::milliseconds(445);
    boost::this_thread::sleep_until(next);

    softcut_white(disp);
    next = next + boost::chrono::milliseconds(445);
    boost::this_thread::sleep_until(next);

    if(i != 8) {
      next = next + boost::chrono::milliseconds(890);
      boost::this_thread::sleep_until(next);
    }
  }
}

void part_end(flipdot::display &disp, std::vector<flipdot::image> const &strips) {
  for(;;) {
    play_strip(disp, strips);
  }
}

int main() {
  auto strips_greetings = load_strips("greetings");
  auto strips_end       = load_strip("end", 0);

  std::ifstream     soundfile("flipdot.wav", std::ios::in | std::ios::binary);
  std::vector<char> sounddata(std::istreambuf_iterator<char>(soundfile),
                              (std::istreambuf_iterator<char>()));

  auto now   = boost::chrono::system_clock::now();
  auto start = boost::chrono::system_clock::now();

  if(sounddata.empty()) {
    throw std::runtime_error("Could not load sound.");
  }

  flipdot::display disp;

  boost::thread sound_thread([&]{
      snd_setup(&sounddata[0], sounddata.size());
    });

  part_intro(disp, start);

  part_plasma(disp);

  now = boost::chrono::system_clock::now();
  std::cerr << now - start << " - " << snd_pos() << '\n';
  start = now;

  part_squishy(disp);

  now = boost::chrono::system_clock::now();
  std::cerr << now - start << " - " << snd_pos() <<  '\n';
  start = now;

  hardcut(disp);
  part_greetings(disp, strips_greetings);

  now = boost::chrono::system_clock::now();
  std::cerr << now - start << " - " << snd_pos() <<  '\n';
  start = now;

  part_cube(disp);

  now = boost::chrono::system_clock::now();
  std::cerr << now - start << " - " << snd_pos() <<  '\n';
  start = now;

  part_procto(disp);

  now = boost::chrono::system_clock::now();
  std::cerr << now - start << " - " << snd_pos() <<  '\n';
  start = now;

  part_end(disp, strips_end);

  sound_thread.join();
}
