#include <flipdot/flipdot.hh>

#include <SFML/Graphics/Image.hpp>

#include <boost/chrono.hpp>
#include <boost/thread.hpp>

namespace {
  double square(double x) { return x * x; }

  void cut_image_part(flipdot::image &fdimg, sf::Image const &sfimg, int off_y) {
    fdimg.clear();

    for(unsigned y = 0; y < fdimg.height(); ++y) {
      for(unsigned x = 0; x < fdimg.width(); ++x) {
        sf::Color c = sfimg.getPixel(x, y + off_y);

        if(std::sqrt(0.241 * square(c.r / 255.) + 0.691 * square(c.g / 255.) + 0.068 * square(c.b / 255.)) < 0.5) {
          fdimg.pixel_black(x, y);
        }
      }
    }
  }

  void cut_image_top(flipdot::image &fdimg, sf::Image const &sfimg, unsigned y_start) {
    fdimg.clear();

    int y_text = y_start;
    y_text -= 16;
    if(y_text > 1) y_text = 1;

    flipdot::render_text(fdimg, 10, y_text    , "Prokto-");
    flipdot::render_text(fdimg, 10, y_text + 8, "Heliose");

    for(unsigned y = y_start; y < fdimg.height(); ++y) {
      for(unsigned x = 0; x < fdimg.width(); ++x) {
        sf::Color c = sfimg.getPixel(x, y - y_start);

        if(std::sqrt(0.241 * square(c.r / 255.) + 0.691 * square(c.g / 255.) + 0.068 * square(c.b / 255.)) < 0.5) {
          fdimg.pixel_black(x, y);
        }
      }
    }
  }
}

void part_procto(flipdot::display &disp) {
  sf::Image sfimg;

  if(!sfimg.loadFromFile("procto.png")) {
    return;
  }

  flipdot::image fdimg;

  if(sfimg.getSize().x != fdimg.width () ||
     sfimg.getSize().y <  fdimg.height()) {
    return;
  }

  boost::posix_time::milliseconds const interval(80);
  auto now  = boost::get_system_time();
  auto next = now + interval;

  for(unsigned y = fdimg.height() - 1; y != 0; --y) {
    fdimg.flush(disp);
    cut_image_top(fdimg, sfimg, y);

    now = next;
    next = now + interval;
    boost::this_thread::sleep(now);
  }

  for(int turn = 0; turn < 3; ++turn) {
    unsigned off_y;

    for(off_y = 0; off_y < sfimg.getSize().y - fdimg.height(); ++off_y) {
      fdimg.flush(disp);
      cut_image_part(fdimg, sfimg, off_y);

      now = next;
      next = now + interval;
      boost::this_thread::sleep(now);
    }

    for(; off_y != 0; --off_y) {
      fdimg.flush(disp);
      cut_image_part(fdimg, sfimg, off_y);

      now = next;
      next = now + interval;
      boost::this_thread::sleep(now);
    }
  }

  fdimg.flush(disp);
}
