#include "point3d.hh"

#include <flipdot/geometry.hh>
#include <cmath>

void move(point3d &pt, double dx, double dy, double dz) {
  pt.x(pt.x() + dx);
  pt.y(pt.y() + dy);
  pt.z(pt.z() + dz);
}

void move(point3d &pt, point3d const &dir) {
  move(pt, dir.x(), dir.y(), dir.z());
}

void rotate(point3d &pt, point3d const &pivot, double anglex, double angley, double anglez) {
  double xd = pt.x() - pivot.x();
  double yd = pt.y() - pivot.y();
  double zd = pt.z() - pivot.z();

  double zx = xd * std::cos(anglez) - yd * std::sin(anglez) - xd;
  double zy = xd * std::sin(anglez) + yd * std::cos(anglez) - yd;

  double yx = (xd + zx) * std::cos(angley) - zd * std::sin(angley) - (xd + zx);
  double yz = (xd + zx) * std::sin(angley) + zd * std::cos(angley) - zd;

  double xy = (yd + zy) * std::cos(anglex) - (zd + yz) * std::sin(anglex) - (yd + zy);
  double xz = (yd + zy) * std::sin(anglex) + (zd + yz) * std::cos(anglex) - (zd + yz);

  double xoff = yx + zx;
  double yoff = zy + xy;
  double zoff = xz + yz;

  move(pt, xoff, yoff, zoff);
}

void rotate(point3d &pt, point3d const &pivot, point3d const &direction) {
  rotate(pt, pivot, direction.x(), direction.y(), direction.z());
}

namespace {
  int round_to_int(double x) { return static_cast<int>(x + 0.5); }
}

void draw_line(flipdot::image &img, point3d const &pt1, point3d const &pt2) {
  flipdot::draw_line(img,
                     round_to_int(pt1.x()),
                     round_to_int(pt1.y()),
                     round_to_int(pt2.x()),
                     round_to_int(pt2.y()));
}

void draw_point(flipdot::image &img, point3d const &pt) {
  img.pixel_black(round_to_int(pt.x()),
                  round_to_int(pt.y()));
}

point3d interpolate(point3d const &pt1, point3d const &pt2, double dist) {
  return point3d(pt1.x() + (pt2.x() - pt1.x()) * dist,
                 pt1.y() + (pt2.y() - pt1.y()) * dist,
                 pt1.z() + (pt2.z() - pt1.z()) * dist);
}
