#include "strips.hh"

#include <flipdot/flipdot.hh>

#include <SFML/Graphics/Image.hpp>

#include <boost/chrono.hpp>
#include <boost/thread.hpp>

#include <cmath>
#include <fstream>
#include <iostream>
#include <numeric>
#include <sstream>
#include <string>

double square(double x) { return x * x; }

flipdot::image load_image(std::string const &filename) {
  sf::Image sfimg;
  if(!sfimg.loadFromFile(filename.c_str())) {
    throw std::logic_error("Could not open file.");
  }

  flipdot::image fdimg(0);

  if(sfimg.getSize().x != fdimg.width() ||
     sfimg.getSize().y != fdimg.height()) {
    std::ostringstream fmt;
    fmt << "Image must be " << fdimg.width() << "x" << fdimg.height() << " pixels.\n";
    throw std::logic_error(fmt.str());
  }

  for(unsigned x = 0; x < fdimg.width(); ++x) {
    for(unsigned y = 0; y < fdimg.height(); ++y) {
      sf::Color c = sfimg.getPixel(x, y);

      if(std::sqrt(0.241 * square(c.r / 255.) + 0.691 * square(c.g / 255.) + 0.068 * square(c.b / 255.)) < 0.5) {
        fdimg.pixel_black(x, y);
      }
    }
  }

  return fdimg;
}

std::vector<flipdot::image> load_strip(std::string const &subdir, int stripnum) {
  std::vector<flipdot::image> result;

  result.reserve(20);

  std::ostringstream fmt;
  fmt << subdir << "/strip-" << stripnum << "-0.png";

  result.push_back(load_image(fmt.str()));

  try {
    for(int i = 1; ; ++i) {
      fmt.clear();
      fmt.str("");
      fmt << subdir << "/strip-" << stripnum << "-" << i << ".png";
      result.push_back(load_image(fmt.str()));
    }
  } catch(...) { }

  return result;
}

std::vector<std::vector<flipdot::image>> load_strips(std::string const &subdir) {
  std::vector<std::vector<flipdot::image>> result;
  result.reserve(100);

  try {
    for(int i = 0; ; ++i) {
      result.push_back(load_strip(subdir, i));
    }
  } catch(...) { }

  return result;
}

void play_strip(flipdot::display &disp, std::vector<flipdot::image> const &strip) {
  if(strip.empty()) return;

  strip[0].flush(disp);
  boost::this_thread::sleep(boost::posix_time::milliseconds(500));

  for(std::size_t i = 1; i < strip.size(); ++i) {
    strip[i].flush(disp);
    boost::this_thread::sleep(boost::posix_time::milliseconds(80));
  }
}

void play_strips(flipdot::display &disp, std::vector<std::vector<flipdot::image>> const &strips) {
  for(auto const &strip : strips) {
    play_strip(disp, strip);
  }
}
