#ifndef INCLUDED_VEC2D_HH
#define INCLUDED_VEC2D_HH

#include <cmath>

class vec2d {
public:
  inline vec2d(double x = 0.0, double y = 0.0) : x_(x), y_(y) { }

  inline double x() const { return x_; }
  inline double y() const { return y_; }

  inline void x(double new_x) { x_ = new_x; }
  inline void y(double new_y) { y_ = new_y; }

  inline vec2d &operator+=(vec2d const &other) {
    x_ += other.x();
    y_ += other.y();
    return *this;
  }

  inline vec2d &operator-=(vec2d const &other) {
    x_ -= other.x();
    y_ -= other.y();
    return *this;
  }

  inline vec2d &operator*=(double d) {
    x_ *= d;
    y_ *= d;
    return *this;
  }

  inline vec2d &operator/=(double d) {
    x_ /= d;
    y_ /= d;
    return *this;
  }

private:
  double x_;
  double y_;
};

inline vec2d  operator-(vec2d lhs, vec2d const &rhs) { lhs -= rhs; return lhs; }
inline vec2d  operator+(vec2d lhs, vec2d const &rhs) { lhs += rhs; return lhs; }

inline vec2d  operator/(vec2d lhs, double rhs) { lhs /= rhs; return lhs; }
inline vec2d  operator*(vec2d lhs, double rhs) { lhs *= rhs; return lhs; }
inline vec2d  operator*(double lhs, vec2d const &rhs) { return rhs * lhs; }
inline double operator*(vec2d const &lhs, vec2d const &rhs) { return lhs.x() * rhs.x() + lhs.y() * rhs.y(); }

inline vec2d  operator-(vec2d const &v) { return vec2d(-v.x(), -v.y()); }

inline double length(vec2d const &vec) { return std::sqrt(vec * vec); }
inline vec2d normal(vec2d const &vec) { return vec2d(vec.y(), -vec.x()); }

#endif
