#include "flipdot.hh"

#include <unistd.h>

#include <algorithm>
#include <functional>
#include <iostream>
#include <stdexcept>

namespace flipdot {
  display::display(std::size_t module_count)
    : module_count_  (module_count),
      display_height_(module_count / DISPLAY_WIDTH),
      linedelim_     (DISPLAY_COLS + 1, '-')
  {
    if(module_count_ % DISPLAY_WIDTH != 0) {
      throw std::invalid_argument("strange display size");
    }

    linedelim_[0] = '+';
    linedelim_ += "+\n";
    std::cout << "\x1b[2J" << std::flush;
  }

  void display::flush(framebuffer const &fb) {
    std::cout << "\x1b[1;1H";

    for(std::size_t box = 0; box < display_height_; ++box) {
      if(box != 0) {
        std::cout << linedelim_;
      }
      std::cout << linedelim_;

      for(std::size_t line = 0; line < MODULE_ROWS; ++line) {
        std::cout << '|';
        for(std::size_t col = 0; col < DISPLAY_COLS && box * DISPLAY_COLS + col < fb.size(); ++col) {
          std::cout << ((fb[box * DISPLAY_COLS + col] & (1 << line)) ? '#' : ' ');
        }
        std::cout << "|\n";
      }

      std::cout << linedelim_;
    }
  }
}
