#ifndef INCLUDED_BONE_GPIO_HH
#define INCLUDED_BONE_GPIO_HH

#include <fstream>
#include <cstdint>
#include <sstream>

#include <unistd.h>

namespace beaglebone {
  namespace gpio {
    enum port_id {
      PORT0 = 0x44E07000,
      PORT1 = 0x4804C000,
      PORT2 = 0x481AC000,
      PORT3 = 0x481AE000
    };

    enum pin_direction {
      PIN_OUTPUT,
      PIN_INPUT
    };

    class port {
    public:
      port(port_id id);

      inline void out(unsigned pin) { oe_ &= ~(1 << pin); }
      inline void in (unsigned pin) { oe_ |=   1 << pin ; }
      inline void pin_mode(unsigned pin, pin_direction mode) {
        if(mode == PIN_OUTPUT) {
          out(pin);
        } else {
          in(pin);
        }
      }

      inline void pin_up  (unsigned pin) { (*this) |=   1 << pin ; }
      inline void pin_down(unsigned pin) { (*this) &= ~(1 << pin); }
      inline void pin_set (unsigned pin, unsigned value) {
        if(value == 0) {
          pin_down(pin);
        } else {
          pin_up  (pin);
        }
      }

      inline port &operator|=(unsigned mask) { location_ |=  mask; return *this; }
      inline port &operator&=(unsigned mask) { location_ &=  mask; return *this; }

    private:
      std::uint32_t volatile &oe_;
      std::uint32_t volatile &location_;
    };

    class pin {
    public:
      pin(port const &port, unsigned number) : p_(port), number_(number) { }

      inline void out () { p_.out(number_); }
      inline void in  () { p_.in (number_); }
      inline void mode(pin_direction dir) { p_.pin_mode(number_, dir); }

      inline void up  () { p_.pin_up  (number_); }
      inline void down() { p_.pin_down(number_); }
      inline void set(unsigned bit) { p_.pin_set(number_, bit); }

    private:
      port     p_;
      unsigned number_;
    };
  }
}

#endif
