#include "iconverter.hh"

#include <iconv.h>

#include <cerrno>
#include <cstddef>
#include <iostream>
#include <string>
#include <stdexcept>
#include <vector>

iconv_error::iconv_error(std::string const &message)
  : std::logic_error(message) { }

iconv_open_error::iconv_open_error(std::string const &tocode, std::string const &fromcode)
  : iconv_error("iconv_open fehlgeschlagen: Umwandlung von " + fromcode + " nach " + tocode + "nicht mglich") { }

iconverter::iconverter(std::string const &tocode,
                       std::string const &fromcode,
                       double expansion_hint)
  : converter_     (iconv_open(tocode  .c_str(),
                               fromcode.c_str())),
    expansion_hint_(expansion_hint) {
  if(converter_ == iconv_t(-1)) {
    throw iconv_open_error(tocode, fromcode);
  }
}

std::string iconverter::operator()(std::string const &text) {
  std::size_t n_in  =                          text.size()                    + 1;
  std::size_t n_out = static_cast<std::size_t>(text.size() * expansion_hint_) + 1; // Grobe Abschtzung der Ausgabelnge
  std::vector<char> outbuffer(n_out);
  std::vector<char> inbuffer;
  char *p_in, *p_out;
  size_t n_converted;

  inbuffer.reserve(n_in);
  inbuffer.assign(text.begin(), text.end());
  inbuffer.push_back('\0');

  p_in  = & inbuffer[0];
  p_out = &outbuffer[0];

  for(;;) {
    errno = 0;
    n_converted = iconv(converter_,
                        &p_in , &n_in,
                        &p_out, &n_out);

    if(n_converted != size_t(-1)) {
      break;
    } else if(errno == E2BIG) {
      size_t n_out_old = p_out - &outbuffer[0];
      n_out += outbuffer.size();
      outbuffer.resize(outbuffer.size() * 2);
      p_out = &outbuffer[n_out_old];
    } else if(errno == EILSEQ) {
      throw iconv_error("Ungltige Multibyte-Sequenz in Eingabe");
    } else if(errno == EINVAL) {
      throw iconv_error("Unvollstndige Multibyte-Sequenz in Eingabe");
    } else {
      throw iconv_error("Unbekannter Umwandlungsfehler");
    }
  }

  return std::string(outbuffer.begin(), outbuffer.begin() + outbuffer.size() - n_out - 1);
}

iconverter::~iconverter() {
  iconv_close(converter_);
}
