#include "flipdot/flipdot.hh"

#include <boost/thread.hpp>
#include <boost/chrono.hpp>

namespace {
  flipdot::sprite const anim_bunny[] = {
    { 7, 5, (std::uint8_t const *) "\x60\x30\xbe\x31\x02" },
    { 7, 5, (std::uint8_t const *) "\x42\x30\xbe\x31\x01" },
    { 7, 5, (std::uint8_t const *) "\x42\x30\xae\x35\x01" },
    { 7, 5, (std::uint8_t const *) "\x60\x30\xae\x35\x02" },
    { 7, 5, (std::uint8_t const *) "\x60\x30\xbe\x31\x01" }
  };

  flipdot::sprite const anim_spiral[] = {
    { 10, 10, (uint8_t const *) "\xff\x07\xd8\x6f\xa1\xb5\xf6\x1a\xe8\xbf\x00\xfe\x0f" },
    { 10, 10, (uint8_t const *) "\xff\x07\xd8\x6f\xa1\xb5\xd6\xda\x69\xb0\x7f\x02\x0c" },
    { 10, 10, (uint8_t const *) "\xff\x07\xd8\x6f\xa1\xb5\xd6\x5a\x6a\xaf\x81\xfe\x0b" },
    { 10, 10, (uint8_t const *) "\xff\x07\xd8\x4f\x61\x35\xd5\x55\x54\x5f\x01\xfd\x07" },
    { 10, 10, (uint8_t const *) "\xff\x07\xd0\x7f\x81\xf5\xd6\x5a\x68\xbf\x01\xfe\x0f" },
    { 10, 10, (uint8_t const *) "\x03\xe4\xdf\x60\xb9\xb5\xd6\x5a\x68\xbf\x01\xfe\x0f" },
    { 10, 10, (uint8_t const *) "\xfd\x17\x58\x6f\xa5\xb5\xd6\x5a\x68\xbf\x01\xfe\x0f" },
    { 10, 10, (uint8_t const *) "\xfe\x0b\xa8\xaf\xa2\xba\xca\x6a\x28\xbf\x01\xfe\x0f" }
  };

  flipdot::sprite const anim_giraffe[] = {
    { 8, 10, (uint8_t const *) "\x00\x10\x60\x80\xff\xc1\x00\x03\x3c\x08" },
    { 8, 10, (uint8_t const *) "\x00\x10\x60\xa0\x7f\xc1\x00\x03\x1c\x88" },
    { 9, 10, (uint8_t const *) "\x00\x10\x68\x90\x3f\xc1\x00\x03\x0c\x48\x00\x02" },
    { 8, 10, (uint8_t const *) "\x00\x10\x60\xb0\x3f\xc1\x00\x03\x0c\xc8" },
    { 8, 10, (uint8_t const *) "\x08\x30\xc0\x81\xfc\xc0\x00\x03\x3c\x08" },
    { 8, 10, (uint8_t const *) "\x00\x10\x60\x80\xff\xc1\x00\x03\x3c\x08" }
  };

  flipdot::sprite const anim_bird[] = {
    { 9, 7, (uint8_t const *) "\x10\x0c\x05\xf7\x4a\xa9\x58\x10" },
    { 9, 7, (uint8_t const *) "\x10\x0c\x05\x87\x47\xa2\x70\x10" }
  };

  flipdot::sprite const anim_snake[] = {
    { 10, 6, (uint8_t const *) "\x10\x86\x83\x30\x04\x83\xa2\x07" },
    { 10, 6, (uint8_t const *) "\x10\x86\x83\x20\x0c\xc1\xa2\x07" },
    { 10, 6, (uint8_t const *) "\x10\x86\x83\x20\x08\x82\xe0\x07" },
    { 10, 6, (uint8_t const *) "\x10\x86\x83\x20\x08\x86\xe1\x07" }
  };
}

template<typename T, std::size_t N>
constexpr std::size_t array_size(T (&)[N]) {
  return N;
}

int main(void) {
  flipdot::display disp;
  flipdot::image img;

  unsigned i_bunny   = 0;
  unsigned i_spiral  = 0;
  unsigned i_giraffe = 0;
  unsigned i_bird    = 0;
  unsigned i_snake   = 0;

  for(int i = 0; i < 100; ++i) {
    img.clear();

    img.blt( 6,  5, anim_bunny[i_bunny]);
    img.blt(42, 17, anim_bunny[i_bunny], flipdot::BLT_MIRRORED);

    img.blt(25, -3, anim_spiral[i_spiral]);
    img.blt(25, 12, anim_spiral[i_spiral], flipdot::BLT_MIRRORED);
    img.blt(25, 27, anim_spiral[i_spiral]);

    img.blt(-4, -5, anim_giraffe[i_giraffe]);
    img.blt(56, -5, anim_giraffe[i_giraffe]);
    img.blt(-2, 28, anim_giraffe[i_giraffe]);
    img.blt(56, 28, anim_giraffe[i_giraffe]);

    img.blt(7, 17, anim_giraffe[i_giraffe], flipdot::BLT_MIRRORED);

    img.blt(42, 3, anim_bird[i_bird]);

    img.blt(41, 25, anim_snake[i_snake]);

    img.flush(disp);

//    boost::this_thread::sleep_for(boost::chrono::milliseconds(300));
    boost::this_thread::sleep(boost::posix_time::milliseconds(300));

    i_bunny   = (i_bunny   + 1) % array_size(anim_bunny  );
    i_spiral  = (i_spiral  + 1) % array_size(anim_spiral );
    i_giraffe = (i_giraffe + 1) % array_size(anim_giraffe);
    i_bird    = (i_bird    + 1) % array_size(anim_bird   );
    i_snake   = (i_snake   + 1) % array_size(anim_snake  );
  }
}
