#include "geometry.hh"
#include <algorithm>

namespace flipdot {
  void draw_line(image &img,
                 int x_from, int y_from,
                 int x_to  , int y_to  ) {
    int dx = std::abs(x_to - x_from);
    int dy = std::abs(y_to - y_from);

    int sx = x_from < x_to ? 1 : -1;
    int sy = y_from < y_to ? 1 : -1;

    int err = dx - dy;

    img.pixel_black(x_from, y_from);

    while(x_from != x_to || y_from != y_to) {
      int8_t e2 = 2 * err;

      if(e2 > -dy) {
        err -= dy;
        x_from += sx;
      }

      if(e2 < dx) {
        err += dx;
        y_from += sy;
      }

      img.pixel_black(x_from, y_from);
    }
  }

  void draw_circle(image &img,
                   int x0, int y0, int r) {
    int f = 1 - r;
    int ddF_x = 1;
    int ddF_y = -2 * r;
    int x = 0;
    int y = r;

    img.pixel_black(x0, y0 + r);
    img.pixel_black(x0, y0 - r);
    img.pixel_black(x0 + r, y0);
    img.pixel_black(x0 - r, y0);

    while(x < y) {
      if(f >= 0) {
        --y;
        ddF_y += 2;
        f += ddF_y;
      }

      ++x;
      ddF_x += 2;
      f += ddF_x;

      img.pixel_black(x0 + x, y0 + y);
      img.pixel_black(x0 - x, y0 + y);
      img.pixel_black(x0 + x, y0 - y);
      img.pixel_black(x0 - x, y0 - y);
      img.pixel_black(x0 + y, y0 + x);
      img.pixel_black(x0 - y, y0 + x);
      img.pixel_black(x0 + y, y0 - x);
      img.pixel_black(x0 - y, y0 - x);
    }
  }
}
