#ifndef INCLUDED_HW_SHIFT_REGISTER_HH
#define INCLUDED_HW_SHIFT_REGISTER_HH

#include <beaglebone/gpio.hh>

#include <cstdint>

namespace hw {
  class shift_register {
  public:
    inline shift_register(beaglebone::gpio::pin const &pin_data,
			  beaglebone::gpio::pin const &pin_clk,
			  unsigned                     length)
      : pin_data(pin_data),
	pin_clk (pin_clk ),
	length  (length  )
    {
      this->pin_data.out();
      this->pin_clk .out();

      this->pin_data.down();
      this->pin_clk .down();
    }

    inline void clock() {
      pin_clk.up  ();
      pin_clk.down();
    }

    inline void push_bit(unsigned bit) {
      pin_data.set(bit);
      clock();
    }

    inline void clear() {
      for(unsigned i = 0; i < length; ++i) {
	push_bit(0);
      }
    }

    inline void set(std::uint16_t data) {
      for(std::uint16_t mask = 0x8000; mask; mask >>= 1) {
	push_bit((data & mask) ? 1 : 0);
      }
    }

  private:
    beaglebone::gpio::pin pin_data;
    beaglebone::gpio::pin pin_clk;
    unsigned              length;
  };
}

#endif
